@@ -28,9 +28,6 @@
  * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-#ifndef lint
-static char rcsid[] = "@(#)$Id: extensions.c,v 1.25 1999/02/26 16:08:50 sob RELEASE sob $";
-#endif /* not lint */
 
 #include "config.h"
 
@@ -44,8 +41,8 @@ static char rcsid[] = "@(#)$Id: extensions.c,v 1.25 1999/02/26 16:08:50 sob RELE
 #endif
 #include <time.h>
 #include <pwd.h>
-#include <setjmp.h>
 #include <grp.h>
+#include <unistd.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -56,53 +53,33 @@ static char rcsid[] = "@(#)$Id: extensions.c,v 1.25 1999/02/26 16:08:50 sob RELE
 #include <sys/statvfs.h>
 #elif defined(HAVE_SYS_VFS)
 #include <sys/vfs.h>
-#elif defined(HAVE_SYS_MOUNT)
-#include <sys/mount.h>
 #endif
 
 #include <arpa/ftp.h>
 
 #include "pathnames.h"
 #include "extensions.h"
-
-#if defined(HAVE_FTW)
-#include <ftw.h>
-#else
 #include "support/ftw.h"
-#endif
 
 #ifdef HAVE_REGEX_H
 #include <regex.h>
 #endif
 
-#if defined(REGEX) && defined(SVR4) && !(defined(__hpux))
+#if defined(REGEX) && defined(SOLARIS_21)
 #include <libgen.h>
 #endif
 
 extern int fnmatch(),
   type,
   transflag,
-  ftwflag,
-  authenticated,
   autospout_free,
   data,
-  pdata,
   anonymous,
   guest;
-
-#ifdef LOG_FAILED
-  extern char the_user[];
-#endif
-
-#ifdef __STDC__
 extern char **ftpglob(register char *v),
-#else
-extern char **ftpglob(),
-#endif
  *globerr,
   remotehost[],
   hostname[],
-  authuser[],
  *autospout,
   Shutdown[];
 
@@ -110,52 +87,32 @@ char shuttime[30],
   denytime[30],
   disctime[30];
 
-#ifdef __STDC__
-extern char *realpath(const char *pathname, char *result);
-#else
-extern char *realpath();
-#endif
-
 #ifndef REGEX
 char *re_comp();
 #elif defined(M_UNIX)
 extern char *regcmp(), *regex();
 #endif
 
-#ifdef __STDC__
 extern FILE *dataconn(char *name, off_t size, char *mode);
-#else
-extern FILE *dataconn();
-#endif
 FILE *dout;
 
 time_t newer_time;
 
 int show_fullinfo;
 
-int
-#ifdef __STDC__
 check_newer(char *path, struct stat *st, int flag)
-#else
-check_newer(path,st,flag)
-char *path;
-struct stat *st;
-int flag;
-#endif
 {
 
     if (st->st_mtime > newer_time) {
         if (show_fullinfo != 0) {
             if (flag == FTW_F || flag == FTW_D) {
-                fprintf(dout, "%s %d %d %s\n", flag == FTW_F ? "F" : "D",
+                fprintf(dout, "%s %d %d %s", flag == FTW_F ? "F" : "D",
                         st->st_size, st->st_mtime, path);
             }
         } else if (flag == FTW_F)
-            fprintf(dout, "%s\n", path);
+            fprintf(dout, "%s", path);
     }
-    /* When an ABOR has been received (which sets ftwflag > 1) return a
-     * non-zero value which causes ftw to stop tree traversal and return. */
-    return (ftwflag > 1 ? 1 : 0);
+    return 0;
 }
 
 #if defined(HAVE_STATVFS)
@@ -170,7 +127,7 @@ char *s;
 
     return(buf.f_bavail * buf.f_frsize / 1024);
 }
-#elif defined(HAVE_SYS_VFS) || defined (HAVE_SYS_MOUNT)
+#elif defined(HAVE_SYS_VFS)
 int getSize(s)
 char *s;
 {
@@ -191,14 +148,8 @@ char *s;
 /* ARGUMENTS : pointer input and output buffers                          */
 /*************************************************************************/
 
-void
-#ifdef __STDC__
+int
 msg_massage(char *inbuf, char *outbuf)
-#else
-msg_massage(inbuf,outbuf)
-char *inbuf;
-char *outbuf;
-#endif
 {
     char *inptr = inbuf;
     char *outptr = outbuf;
@@ -206,40 +157,36 @@ char *outbuf;
     time_t curtime;
     int limit;
     extern struct passwd *pw;
-    struct aclmember *entry;
+    struct aclmember *entry = NULL;
 
+    (void) time(&curtime);
     (void) acl_getclass(buffer);
+
     limit = acl_getlimit(buffer, NULL);
 
     while (*inptr) {
         if (*inptr != '%')
             *outptr++ = *inptr;
         else {
-            entry = NULL;
             switch (*++inptr) {
             case 'E':
                 if ( (getaclentry("email", &entry)) && ARG0 )
                     sprintf(outptr, "%s", ARG0); 
-                else
-                    *outptr = '\0';
                 break;
             case 'N': 
                 sprintf(outptr, "%d", acl_countusers(buffer)); 
                 break; 
             case 'M':
-                if (limit == -1)
-                    strcpy(outptr,"unlimited");
-                else
-                    sprintf(outptr, "%d", limit);
+                sprintf(outptr, "%d", limit);
                 break;
+
             case 'T':
- 	        (void) time(&curtime);
                 strncpy(outptr, ctime(&curtime), 24);
                 *(outptr + 24) = '\0';
                 break;
 
             case 'F':
-#if defined(HAVE_STATVFS) || defined(HAVE_SYS_VFS) || defined(HAVE_SYS_MOUNT)
+#if defined(HAVE_STATVFS) || defined(HAVE_SYS_VFS)
                 sprintf(outptr, "%lu", getSize("."));
 #endif
                 break;
@@ -261,13 +208,7 @@ char *outbuf;
                 break;
 
             case 'U':
-#ifdef LOG_FAILED
-                strcpy (outptr, the_user);
-#else /* LOG_FAILED */
-                strcpy(outptr,
-                    (pw == NULL) ? "[unknown]" : pw->pw_name);
-#endif /* LOG_FAILED */
-
+                strcpy(outptr, pw->pw_name);
                 break;
 
             case 's':
@@ -285,13 +226,6 @@ char *outbuf;
                 *(outptr + 24) = '\0';
                 break;
 
-/* KH : cookie %u for RFC931 name */
-            case 'u':
-                if (authenticated) strncpy(outptr, authuser, 24);
-                else strcpy(outptr,"[unknown]");
-                *(outptr + 24) = '\0'; 
-                break;
-
             case '%':
                 *outptr++ = '%';
                 *outptr = '\0';
@@ -319,12 +253,7 @@ char *outbuf;
 /*************************************************************************/
 
 int
-#ifdef __STDC__
 cwd_beenhere(int dircode)
-#else
-cwd_beenhere(dircode)
-int dircode;
-#endif
 {
     struct dirlist {
         struct dirlist *next;
@@ -362,14 +291,9 @@ int dircode;
 /* PURPOSE   : Display a banner on the user's terminal before login      */
 /* ARGUMENTS : reply code to use                                         */
 /*************************************************************************/
- 
+
 void
-#ifdef __STDC__
 show_banner(int msgcode)
-#else
-show_banner(msgcode)
-int msgcode;
-#endif
 {
     char *crptr,
       linebuf[1024],
@@ -377,43 +301,24 @@ int msgcode;
     struct aclmember *entry = NULL;
     FILE *infile;
 
-#ifdef VIRTUAL
-    extern int virtual_mode;
-    extern char virtual_banner[];
-
-    if (virtual_mode) {
-        infile = fopen(virtual_banner, "r");
-	if (infile) {
- 	    while (fgets(linebuf, 255, infile) != NULL) {
-	           if ((crptr = strchr(linebuf, '\n')) != NULL)
-		        *crptr = '\0';
-		   msg_massage(linebuf, outbuf);
-		   lreply(msgcode, "%s", outbuf);
-		 }
-	    fclose(infile);
-	    lreply(msgcode, "");
-	  }
-      }
-    else {
-#endif
-      /* banner <path> */
-      while (getaclentry("banner", &entry)) {
-	    infile = fopen(ARG0, "r");
-	    if (infile) {
-	        while (fgets(linebuf, 255, infile) != NULL) {
-		  if ((crptr = strchr(linebuf, '\n')) != NULL)
-		    *crptr = '\0';
-		  msg_massage(linebuf, outbuf);
-		  lreply(msgcode, "%s", outbuf);
-		}
-		fclose(infile);
-		lreply(msgcode, "");
-	    }
-	 }
-#ifdef VIRTUAL
+    /* banner <path> */
+    while (getaclentry("banner", &entry)) {
+        if (ARG0 && strlen(ARG0) > 0) {
+            infile = fopen(ARG0, "r");
+            if (infile) {
+                while (fgets(linebuf, 255, infile) != NULL) {
+                    if ((crptr = strchr(linebuf, '\n')) != NULL)
+                        *crptr = '\0';
+                    msg_massage(linebuf, outbuf);
+                    lreply(msgcode, "%s", outbuf);
+                }
+                fclose(infile);
+                lreply(msgcode, "");
+            }
+        }
     }
-#endif
-  }
+}
+
 /*************************************************************************/
 /* FUNCTION  : show_message                                              */
 /* PURPOSE   : Display a message on the user's terminal if the current   */
@@ -422,13 +327,7 @@ int msgcode;
 /*************************************************************************/
 
 void
-#ifdef __STDC__
 show_message(int msgcode, int mode)
-#else
-show_message(msgcode,mode)
-int msgcode;
-int mode;
-#endif
 {
     char *crptr,
       linebuf[1024],
@@ -466,7 +365,7 @@ int mode;
             }
         if (mode == C_WD && ARG1 && !strncasecmp(ARG1, "cwd=", 4) &&
             (!strcmp((ARG1) + 4, cwd) || *(ARG1 + 4) == '*' ||
-            !fnmatch((ARG1) + 4, cwd, FNM_PATHNAME)))
+            fnmatch((ARG1) + 4, cwd, FNM_PATHNAME)))
             if (!ARG2)
                 show++;
             else {
@@ -474,7 +373,7 @@ int mode;
                     if (strcasecmp(class, ARG[which]) == 0)
                         show++;
             }
-        if (show && (int)strlen(ARG0) > 0) {
+        if (show && strlen(ARG0) > 0) {
             infile = fopen(ARG0, "r");
             if (infile) {
                 while (fgets(linebuf, 255, infile) != NULL) {
@@ -498,16 +397,9 @@ int mode;
 /*************************************************************************/
 
 void
-#ifdef __STDC__
 show_readme(int code, int mode)
-#else
-show_readme(code,mode)
-int code;
-int mode;
-#endif
 {
     char **filelist,
-      **sfilelist,
       class[MAXPATHLEN],
       cwd[MAXPATHLEN];
     int show,
@@ -545,7 +437,7 @@ int mode;
             }
         if (mode == C_WD && ARG1 && !strncasecmp(ARG1, "cwd=", 4)
             && (!strcmp((ARG1) + 4, cwd) || *(ARG1 + 4) == '*' ||
-                !fnmatch((ARG1) + 4, cwd, FNM_PATHNAME)))
+                fnmatch((ARG1) + 4, cwd, FNM_PATHNAME)))
             if (!ARG2)
                 show++;
             else {
@@ -556,27 +448,25 @@ int mode;
         if (show) {
             globerr = NULL;
             filelist = ftpglob(ARG0);
-            sfilelist = filelist;  /* save to free later */
             if (!globerr) {
                 while (filelist && *filelist) {
                    errno = 0;
-                   if (!stat(*filelist, &buf) &&
-                       (buf.st_mode & S_IFMT) == S_IFREG) {
+                   if (!stat(*filelist, &buf)) {
                        lreply(code, "Please read the file %s", *filelist);
                        (void) time(&clock);
                        tp = localtime(&clock);
                        days = 365 * tp->tm_year + tp->tm_yday;
-                       tp = localtime((time_t *)&buf.st_mtime);
+                       tp = localtime(&buf.st_mtime);
                        days -= 365 * tp->tm_year + tp->tm_yday;
 /*
                        if (days == 0) {
                          lreply(code, "  it was last modified on %.24s - Today",
-                           ctime((time_t *)&buf.st_mtime));
+                           ctime(&buf.st_mtime));
                        } else {
 */
                          lreply(code, 
                            "  it was last modified on %.24s - %d day%s ago",
-                           ctime((time_t *)&buf.st_mtime), days, days == 1 ? "" : "s");
+                           ctime(&buf.st_mtime), days, days == 1 ? "" : "s");
 /*
                        }
 */
@@ -584,10 +474,6 @@ int mode;
                    filelist++;
                 }
             }
-            if (sfilelist) {
-                blkfree(sfilelist);
-                free((char *) sfilelist);
-            }
         }
     }
 }
@@ -601,13 +487,7 @@ int mode;
 /*************************************************************************/
 
 int
-#ifdef __STDC__
 deny_badasciixfer(int msgcode, char *filepath)
-#else
-deny_badasciixfer(msgcode,filepath)
-int msgcode;
-char *filepath;
-#endif
 {
 
     if (type == TYPE_A && !*filepath) {
@@ -626,13 +506,7 @@ char *filepath;
 /*************************************************************************/
 
 int
-#ifdef __STDC__
-is_shutdown(int quiet, int new)
-#else
-is_shutdown(quiet, new)
-int quiet;
-int new;
-#endif
+is_shutdown(int quiet)
 {
     static struct tm tmbuf;
     static struct stat s_last;
@@ -640,7 +514,7 @@ int new;
       shut,
       deny,
       disc;
-    static int valid;
+
     static char text[2048];
 
     struct stat s_cur;
@@ -660,7 +534,6 @@ int new;
 
     if (s_last.st_mtime != s_cur.st_mtime) {
         s_last = s_cur;
-        valid = 0;
 
         fp = fopen(Shutdown, "r");
         if (fp == NULL)
@@ -668,10 +541,8 @@ int new;
         fgets(buf, sizeof(buf), fp);
         if (sscanf(buf, "%d %d %d %d %d %d %d", &tmbuf.tm_year, &tmbuf.tm_mon,
         &tmbuf.tm_mday, &tmbuf.tm_hour, &tmbuf.tm_min, &deny, &disc) != 7) {
-            (void) fclose(fp);
             return (0);
         }
-        valid = 1;
         deny_off = 3600 * (deny / 100) + 60 * (deny % 100);
         disc_off = 3600 * (disc / 100) + 60 * (disc % 100);
 
@@ -696,19 +567,12 @@ int new;
 
         (void) fclose(fp);
     }
-    if (!valid)
-        return (0);
-
     /* if last == 0, then is_shutdown() only called with quiet == 1 so far */
     if (last == 0 && !quiet) {
         autospout = text;       /* warn them for the first time */
         autospout_free = 0;
         last = curtime;
     }
-    /* if a new connection and past deny time, tell caller to drop 'em */
-    if (new && curtime > deny)
-        return (1);
-
     /* if past disconnect time, tell caller to drop 'em */
     if (curtime > disc)
         return (1);
@@ -746,15 +610,7 @@ int new;
     return (0);
 }
 
-void
-#ifdef __STDC__
 newer(char *date, char *path, int showlots)
-#else
-newer(date,path,showlots)
-char *date;
-char *path;
-int showlots;
-#endif
 {
     struct tm tm;
 
@@ -767,14 +623,10 @@ int showlots;
         tm.tm_isdst = -1;
         newer_time = mktime(&tm);
         dout = dataconn("file list", (off_t) - 1, "w");
-
+        /* dout = dataconn("file list", (off_t)-1, "w", 0); */
+        transflag++;
         if (dout != NULL) {
-            /* As ftw allocates storage it needs a chance to cleanup, setting
-             * ftwflag prevents myoob from calling longjmp, incrementing
-             * ftwflag instead which causes check_newer to return non-zero
-             * which makes ftw return. */
-            ftwflag = 1;
-            transflag++;
+
             show_fullinfo = showlots;
 #if defined(HAVE_FTW)
             ftw(path, check_newer, -1);
@@ -782,31 +634,21 @@ int showlots;
             treewalk(path, check_newer, -1, NULL);
 #endif
 
-            /* don't send a reply if myoob has already replied */
-            if (ftwflag == 1) {
-                if (ferror(dout) != 0)
-                    perror_reply(550, "Data connection");
-                else
-                    reply(226, "Transfer complete.");
-            }
+            if (ferror(dout) != 0)
+                perror_reply(550, "Data connection");
+            else
+                reply(226, "Transfer complete.");
 
             (void) fclose(dout);
             data = -1;
-            pdata = -1;
-            transflag = 0;
-            ftwflag = 0;
         }
     } else
         reply(501, "Bad DATE format");
+    transflag = 0;
 }
 
 int
-#ifdef __STDC__
 type_match(char *typelist)
-#else
-type_match(typelist)
-char *typelist;
-#endif
 {
     if (anonymous && strcasestr(typelist, "anonymous"))
         return (1);
@@ -819,26 +661,16 @@ char *typelist;
 }
 
 int
-#ifdef __STDC__
 path_compare(char *p1, char *p2)
-#else
-path_compare(p1,p2)
-char *p1;
-char *p2;
-#endif
 {
-    if ( fnmatch(p1, p2, NULL) == 0 ) /* 0 means they matched */
+    if ( fnmatch(p1, p2, NULL) != 0 )
         return(strlen(p1));
     else
-        return(-2);
+        return(-1);
 }
 
 void
-#ifdef __STDC__
 expand_id(void)
-#else
-expand_id()
-#endif
 {
     struct aclmember *entry = NULL;
     struct passwd *pwent;
@@ -859,18 +691,12 @@ expand_id()
             else        sprintf(buf, "%d", 0);
             ARG4 = (char *) malloc(strlen(buf) + 1);
             strcpy(ARG4, buf);
-	    endgrent();
         }
     }
 }
 
 int
-#ifdef __STDC__
 fn_check(char *name)
-#else
-fn_check(name)
-char *name;
-#endif
 {
   /* check to see if this is a valid file name... path-filter <type>
    * <message_file> <allowed_charset> <disallowed> */
@@ -902,14 +728,14 @@ char *name;
 
           /* is it in the allowed character set? */
 #if defined(REGEXEC)
-          if (regcomp(&regexbuf, ARG2, REG_EXTENDED) != 0) {
-              reply(550, "REGEX error");
+          if (regcomp(&regexbuf, ARG2, REG_EXTENDED|REG_ICASE) != 0) {
+              reply(553, "REGEX error");
 #elif defined(REGEX)
           if ((sp = regcmp(ARG2, (char *) 0)) == NULL) {
-              reply(550, "REGEX error");
+              reply(553, "REGEX error");
 #else
           if ((sp = re_comp(ARG2)) != 0) {
-              perror_reply(550, sp);
+              perror_reply(553, sp);
 #endif
               return(0);
           }
@@ -926,8 +752,8 @@ char *name;
 #else
           if ((re_exec(path)) != 1) {
 #endif
-              pr_mesg(550, ARG1);
-              reply(550, "%s: Permission denied. (Filename (accept))", name);
+              pr_mesg(553, ARG1);
+              reply(553, "%s: Permission denied. (Filename (accept))", name);
               return(0);
           }
           /* is it in any of the disallowed regexps */
@@ -936,14 +762,15 @@ char *name;
               /* ARGj == entry->arg[j] */
               if (entry->arg[j]) {
 #if defined(REGEXEC)
-                  if (regcomp(&regexbuf, entry->arg[j], REG_EXTENDED) != 0) {
-                      reply(550, "REGEX error");
+                  if (regcomp(&regexbuf, entry->arg[j], 
+                        REG_EXTENDED|REG_ICASE) !=0) {
+                      reply(553, "REGEX error");
 #elif defined(REGEX)
                   if ((sp = regcmp(entry->arg[j], (char *) 0)) == NULL) {
-                      reply(550, "REGEX error");
+                      reply(553, "REGEX error");
 #else
                   if ((sp = re_comp(entry->arg[j])) != 0) {
-                      perror_reply(550, sp);
+                      perror_reply(553, sp);
 #endif
                       return(0);
                   }
@@ -960,8 +787,8 @@ char *name;
 #else
                   if ((re_exec(path)) == 1) {
 #endif
-                      pr_mesg(550, ARG1);
-                      reply(550, "%s: Permission denied. (Filename (deny))", name);
+                      pr_mesg(553, ARG1);
+                      reply(553, "%s: Permission denied. (Filename (deny))", name);
                       return(0);
                   }
               }
@@ -972,15 +799,7 @@ char *name;
 }
 
 int
-#ifdef __STDC__
 dir_check(char *name, uid_t *uid, gid_t *gid, int *valid)
-#else
-dir_check(name,uid,gid,valid)
-char *name;
-uid_t *uid;
-gid_t *gid;
-int *valid;
-#endif
 {
   struct aclmember *entry = NULL;
 
@@ -990,25 +809,26 @@ int *valid;
        *ap3 = NULL,
        *ap4 = NULL,
        *ap6 = NULL;
-  char cwdir[MAXPATHLEN];
-  char path[MAXPATHLEN];
+  char cwdir[BUFSIZ];
+  char path[BUFSIZ];
   char *sp;
   extern struct passwd *pw;
 
   *valid = 0;
-  /* what's our current directory? */
 
   strcpy(path, name);
   if (sp = strrchr(path, '/'))  *sp = '\0';
   else strcpy(path, ".");
 
   if ((realpath(path, cwdir)) == NULL) {
-    perror_reply(550, "Could not determine cwdir");
-    return(-1);
+      perror_reply(553, "Could not determine cwdir");
+      return(0);
   }
+
+  i = match_value;
   while (getaclentry("upload", &entry) && ARG0 && ARG1 && ARG2 != NULL) {
-      if ( (!strcmp(ARG0, pw->pw_dir)) &&
-           ((i = path_compare(ARG1, cwdir)) >= match_value) ) {
+      if ( (!strcmp(ARG0, pw->pw_dir))  &&
+		   ((i = path_compare(ARG1, cwdir)) >= match_value) ) {
           match_value = i;
           ap2 = ARG2;
           if (ARG3)  ap3 = ARG3;
@@ -1019,35 +839,29 @@ int *valid;
           else       ap6 = NULL;
       }
   }
-  if ( (ap3 && !strcasecmp(ap3, "nodirs")) ||
-       (ap6 && !strcasecmp(ap6, "nodirs")) ) {
-      reply(550, "%s: Permission denied. (Upload dirs)", name);
+
+  if ( ((ap2 && !strcasecmp(ap2, "no")) && (ap3 && strcasecmp(ap3, "dirs"))) || 
+        (ap3 && !strcasecmp(ap3, "nodirs")) ||
+		(ap6 && !strcasecmp(ap6, "nodirs")) ) {
+      reply(530, "%s: Permission denied.  (Upload)", name);
       return(0);
   }
+
   if (ap3)
      *uid = atoi(ap3);    /* the uid  */
   if (ap4) {
-     *gid = atoi(ap4);    /* the gid */
+     *gid = atoi(ap4);    /* the gid  */
      *valid = 1;
    }
   return(1);
 }
 
 int
-#ifdef __STDC__
 upl_check(char *name, uid_t *uid, gid_t *gid, int *f_mode, int *valid)
-#else
-upl_check(name,uid,gid,f_mode,valid)
-char *name;
-uid_t *uid;
-gid_t *gid;
-int *f_mode;
-int *valid;
-#endif
 {
   int  match_value = -1;
-  char cwdir[MAXPATHLEN];
-  char path[MAXPATHLEN];
+  char cwdir[BUFSIZ];
+  char path[BUFSIZ];
   char *sp;
   int  i;
 
@@ -1076,8 +890,11 @@ int *valid;
       /* we are doing a "best match"... ..so we keep track of what "match
        * value" we have received so far... */
 
+      entry = NULL;
+      match_value = -1;
+      i = match_value;
       while (getaclentry("upload", &entry) && ARG0 && ARG1 && ARG2 != NULL) {
-          if ( (!strcmp(ARG0, pw->pw_dir)) &&
+          if ( (!strcmp(ARG0, pw->pw_dir))  &&
 		       ((i = path_compare(ARG1, cwdir)) >= match_value) ) {
               match_value = i;
               ap1 = ARG1;
@@ -1089,7 +906,7 @@ int *valid;
               if (ARG5) ap5 = ARG5;
               else      ap5 = NULL;
           }
-	}
+      }
 
       if (ap3 && ( (!strcasecmp("dirs",ap3)) || (!strcasecmp("nodirs", ap3)) ))
         ap3 = NULL;
@@ -1120,12 +937,7 @@ int *valid;
 }
 
 int
-#ifdef __STDC_
 del_check(char *name)
-#else
-del_check(name)
-char *name;
-#endif
 {
   int pdelete = 1;
   struct aclmember *entry = NULL;
@@ -1135,11 +947,6 @@ char *name;
           if (*ARG0 == 'n')
               pdelete = 0;
   }
-  
-/* H* fix: no deletion, period. You put a file here, I get to look at it. */
-#ifdef PARANOID
-  pdelete = 0;
-#endif
 
   if (!pdelete) {
       reply(553, "%s: Permission denied. (Delete)", name);
@@ -1149,50 +956,53 @@ char *name;
   }
 }
 
-/* The following is from the Debian add-ons. */
+#define basename(x) (strrchr(x,'/')?1+strrchr(x,'/'):x)
 
-#define lbasename(x) (strrchr(x,'/')?1+strrchr(x,'/'):x)
+struct nr {
+	struct nr *next;
+	char *entry;
+};
+
+static struct nr *noretrieve_entry = NULL;
 
 int
-#ifdef __STDC__
 checknoretrieve (char *name)
-#else
-checknoretrieve (name)
-char *name;
-#endif
 {
-  char cwd[MAXPATHLEN+1], realwd[MAXPATHLEN+1], realname[MAXPATHLEN+1];
+  char cwd[MAXPATHLEN], realwd[MAXPATHLEN], realname[MAXPATHLEN];
   int i;
   struct aclmember *entry = NULL;
+  struct nr *base;
 
-  if (name == (char *)NULL || *name == '\0')
-    return 0; 
-
-#ifdef HAVE_GETCWD
-  (void)getcwd (cwd, MAXPATHLEN);
-#else
-  (void)getwd (cwd);
-#endif
+  if (!name || *name == (char) NULL)
+    return 0;
 
+  (void) getwd (cwd);
   realpath (cwd, realwd);
   realpath (name, realname);
 
-   while (getaclentry("noretrieve", &entry)) {
-        if (ARG0 == (char *)NULL)
-            continue;
-	for (i = 0; i< MAXARGS && 
-	     (entry->arg[i] != (char *)NULL) && (*(entry->arg[i]) !='\0'); i++)
-	  if (strcmp (((*(entry->arg[i]) == '/') ? realname : 
-			lbasename (realname)), entry->arg[i]) == 0)
-	  {
-	    reply (550, "%s is marked unretrievable", entry->arg[i]);
-	    return 1;
-	  }
-      }
-   return 0;
+  if (noretrieve_entry == NULL)
+    {
+      base = noretrieve_entry = (struct nr *) malloc (sizeof (struct nr));
+      while (getaclentry ("noretrieve", &entry))
+	{
+	  if (!ARG0)
+	    continue;
+	  for (i = 0; i < MAXARGS && entry->arg[i] && *entry->arg[i]; i++)
+	    {
+	      base->entry = (char *) strdup (entry->arg[i]);
+	      base = base->next = (struct nr *) malloc (sizeof (struct nr));
+	      base->next = (struct nr *) NULL;
+	    }
+	}
+    }
+  for (base = noretrieve_entry; base; base = base->next)
+    {
+      if (!strcmp (*base->entry == '/' ? realname : basename (realname),
+		   base->entry))
+	{
+	  reply (550, "%s is marked unretrievable", base->entry);
+	  return 1;
+	}
+    }
+  return 0;
 }
-
-
-
-
-
