@@ -560,10 +560,10 @@ SIGNAL_TYPE
 randomsig(int sig)
 {
     syslog(LOG_ERR, "exiting on signal %d", sig);
-    chdir("/etc/tmp");
+    chdir("/tmp");
     signal(SIGIOT, SIG_DFL);
     signal(SIGILL, SIG_DFL);
-    abort();
+    exit(1);
     /* dologout(-1); *//* NOTREACHED */
 }
 
@@ -999,7 +999,7 @@ pass(char *passwd)
         }
 #endif
 
-        *guestpw = NULL;
+        *guestpw = '\0';
         if (pw == NULL)
             salt = "xx";
         else
@@ -1191,7 +1191,8 @@ pass(char *passwd)
 #ifdef SETPROCTITLE
         sprintf(proctitle, "%s: anonymous/%.*s", remotehost,
                     sizeof(proctitle) - sizeof(remotehost) -
-                    sizeof(": anonymous/"), passwd);
+                    sizeof(": anonymous/"),
+                    strchr(passwd, '@')? passwd : "<bad password>");
         setproctitle("%s", proctitle);
 #endif /* SETPROCTITLE */
         if (logging)
@@ -1223,11 +1224,11 @@ opt_string(int options)
     static char buf[100];
     char *ptr = buf;
 
-    if ((options & O_COMPRESS) != NULL)
+    if ((options & O_COMPRESS) != 0)
         *ptr++ = 'C';
-    if ((options & O_TAR) != NULL)
+    if ((options & O_TAR) != 0)
         *ptr++ = 'T';
-    if ((options & O_UNCOMPRESS) != NULL)
+    if ((options & O_UNCOMPRESS) != 0)
         *ptr++ = 'U';
     if (options == 0)
         *ptr++ = '_';
@@ -1247,8 +1248,13 @@ retrieve(char *cmd, char *name)
     static char *logname;
     char namebuf[MAXPATHLEN];
     struct convert *cptr;
+    extern int checknoretrieve (char *);
+    int stat_ret;
 
-    if (!cmd && stat(name, &st)) {
+    if (!cmd && !(stat_ret = stat(name, &st)) && checknoretrieve(name))
+	return;
+
+    if (!cmd && stat_ret) {
         char fnbuf[MAXPATHLEN],
          *ptr;
 
@@ -1322,6 +1328,8 @@ retrieve(char *cmd, char *name)
                 }
             }
 
+	    if (checknoretrieve(fnbuf)) return;
+
             if (S_ISREG(st.st_mode) && !(cptr->types & T_REG)) {
                 (void) reply(550, "Cannot %s plain files.", cptr->name);
                 return;
@@ -1339,7 +1347,7 @@ retrieve(char *cmd, char *name)
             strcpy(namebuf, cptr->external_cmd);
             if ((ptr = strchr(namebuf, ' ')) != NULL)
                 *ptr = '\0';
-            if (stat(namebuf, &st) != NULL) {
+            if (stat(namebuf, &st) != 0) {
                 syslog(LOG_ERR, "external command %s not found",
                        namebuf);
                 (void) reply(550,
@@ -1807,9 +1815,19 @@ send_data(FILE *instr, FILE *outstr, off_t blksize)
         }
         netfd = fileno(outstr);
         filefd = fileno(instr);
+#ifdef __linux__
+	while ((cnt = read(filefd, buf, (u_int)blksize)) > 0)
+	{
+	int outcnt=0, newcnt=0;
+	while ((outcnt=write(netfd, buf+newcnt, cnt-newcnt))!= cnt-newcnt)
+		newcnt+=outcnt;
+	byte_count += cnt;				
+	}
+#else
         while ((cnt = read(filefd, buf, (u_int) blksize)) > 0 &&
                write(netfd, buf, cnt) == cnt)
             byte_count += cnt;
+#endif
         transflag = 0;
         (void) free(buf);
         if (cnt != 0) {
