@@ -32,7 +32,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$Id: glob.c,v 1.14 1999/02/26 16:08:50 sob RELEASE sob $ from glob.c 5.9 (Berkeley) 2/25/91";
+static char sccsid[] = "@(#)glob.c	5.9 (Berkeley) 2/25/91";
 #endif /* not lint */
 
 /*
@@ -54,6 +54,7 @@ static char sccsid[] = "@(#)$Id: glob.c,v 1.14 1999/02/26 16:08:50 sob RELEASE s
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #define	QUOTE 0200
 #define	TRIM 0177
@@ -65,11 +66,7 @@ static	char **gargv;		/* Pointer to the (stack) arglist */
 static	int gargc;		/* Number args in gargv */
 static	int gnleft;
 static	short gflag;
-#ifdef __STDC__
-static int tglob(register char);
-#else
-static int tglob();
-#endif
+static	int tglob();
 char	**ftpglob();
 char	*globerr;
 char	*home;
@@ -78,7 +75,7 @@ char	*strspl();
 static	char *strend();
 char	**copyblk();
 
-static void acollect(), collect(), expand(), Gcat();
+static void acollect(), addpath(), collect(), expand(), Gcat();
 static void ginit(), matchdir(), rscan(), sort();
 static int amatch(), execbrc(), match();
 
@@ -91,40 +88,15 @@ static	int globbed;
 static	char *entp;
 static	char **sortbas;
 
-#ifdef __STDC__
-static void addpath(char); 
-void blkfree(char **);
-int letter(register char);
-int digit(register char);
-int gethdir(char *);
-int any(register int, register char *);
-#else
-static void addpath();
-void blkfree();
-int letter();
-int digit();
-int gethdir();
-int any();
-#endif
-
 char **
-#ifdef __STDC__
-ftpglob(register char *v)
-#else
 ftpglob(v)
-register char *v;
-#endif
+	register char *v;
 {
 	char agpath[BUFSIZ];
 	char *agargv[GAVSIZ];
 	char *vv[2];
-	if (strstr(v, "../*/..") != (char *)NULL){
-		globerr = "Arguments too long";
-		return(0);
-	}
 	vv[0] = v;
-	vv[1] = NULL;
-	globerr = NULL;
+	vv[1] = 0;
 	gflag = 0;
 	rscan(vv, tglob);
 	if (gflag == 0) {
@@ -132,7 +104,7 @@ register char *v;
 		return (copyblk(vv));
 	}
 
-	globerr = NULL;
+	globerr = 0;
 	gpath = agpath; gpathp = gpath; *gpathp = 0;
 	lastgpathp = &gpath[sizeof agpath - 2];
 	ginit(agargv); globcnt = 0;
@@ -145,12 +117,8 @@ register char *v;
 }
 
 static void
-#ifdef __STDC__
-ginit(char ** agargv)
-#else
 ginit(agargv)
-char ** agargv;
-#endif
+	char **agargv;
 {
 
 	agargv[0] = 0; gargv = agargv; sortbas = agargv; gargc = 0;
@@ -158,12 +126,8 @@ char ** agargv;
 }
 
 static void
-#ifdef __STDC__
-collect(register char *as)
-#else
 collect(as)
-register char *as;
-#endif
+	register char *as;
 {
 	if (eq(as, "{") || eq(as, "{}")) {
 		Gcat(as, "");
@@ -173,12 +137,8 @@ register char *as;
 }
 
 static void
-#ifdef __STDC__
-acollect(register char *as)
-#else
 acollect(as)
-register char *as;
-#endif
+	register char *as;
 {
 	register int ogargc = gargc;
 
@@ -206,12 +166,8 @@ sort()
 }
 
 static void
-#ifdef __STDC__
-expand(char *as)
-#else
 expand(as)
-char *as;
-#endif
+	char *as;
 {
 	register char *cs;
 	register char *sgpathp, *oldcs;
@@ -263,12 +219,8 @@ endit:
 }
 
 static void
-#ifdef __STDC__
-matchdir(char *pattern)
-#else
 matchdir(pattern)
-char *pattern;
-#endif
+	char *pattern;
 {
 	struct stat stb;
 
@@ -286,11 +238,7 @@ char *pattern;
 			return;
 		goto patherr2;
 	}
-#ifdef HAVE_DIRFD
-       if (fstat(dirfd(dirp), &stb) < 0)
-#else /* HAVE_DIRFD */
-        if (fstat(dirp->dd_fd, &stb) < 0)
-#endif /* HAVE_DIRFD */
+	if (fstat(dirp->dd_fd, &stb) < 0)
 		goto patherr1;
 	if (!isdir(stb)) {
 		errno = ENOTDIR;
@@ -314,12 +262,8 @@ patherr2:
 }
 
 static int
-#ifdef __STDC__
-execbrc(char *p, char *s)
-#else
 execbrc(p, s)
-char *p, *s;
-#endif
+	char *p, *s;
 {
 	char restbuf[BUFSIZ + 2];
 	register char *pe, *pm, *pl;
@@ -398,12 +342,8 @@ doit:
 }
 
 static int
-#ifdef __STDC__
-match(char *s, char *p)
-#else
-match(s,p)
-char *s, *p;
-#endif
+match(s, p)
+	char *s, *p;
 {
 	register int c;
 	register char *sentp;
@@ -420,12 +360,8 @@ char *s, *p;
 }
 
 static int
-#ifdef __STDC__
-amatch(char *s, char *p)
-#else
 amatch(s, p)
-char *s, *p;
-#endif
+	register char *s, *p;
 {
 	register int scc;
 	int ok, lc;
@@ -513,13 +449,9 @@ slash:
 	}
 }
 
-static int
-#ifdef __STDC__
-Gmatch(register char *s, register char *p)
-#else
-Gmatch(s,p)
-register char *s, *p;
-#endif
+static
+Gmatch(s, p)
+	register char *s, *p;
 {
 	register int scc;
 	int ok, lc;
@@ -578,12 +510,8 @@ register char *s, *p;
 }
 
 static void
-#ifdef __STDC__
-Gcat(register char *s1, register char *s2)
-#else
-Gcat(s1,s2)
-register char *s1, *s2;
-#endif
+Gcat(s1, s2)
+	register char *s1, *s2;
 {
 	register int len = strlen(s1) + strlen(s2) + 1;
 
@@ -598,12 +526,8 @@ register char *s1, *s2;
 }
 
 static void
-#ifdef __STDC__
-addpath(char c)
-#else
 addpath(c)
-char c;
-#endif
+	char c;
 {
 
 	if (gpathp >= lastgpathp)
@@ -615,72 +539,71 @@ char c;
 }
 
 static void
-#ifdef __STDC__
-rscan(register char **t, int (*f)())
-#else
-rscan(t,f)
-register char **t;
-int (*f)();
-#endif
+rscan(t, f)
+	register char **t;
+	int (*f)();
 {
 	register char *p, c;
 
 	while (p = *t++) {
-		if (*p == '~')
-			gflag |= 2;
-		else if (eq(p, "{") || eq(p, "{}"))
-			continue;
+		if (f == tglob)
+			if (*p == '~')
+				gflag |= 2;
+			else if (eq(p, "{") || eq(p, "{}"))
+				continue;
 		while (c = *p++)
 			(*f)(c);
 	}
 }
-static int
-#ifdef __STDC__
-tglob(register char c)
-#else
+/*
+static
+scan(t, f)
+	register char **t;
+	int (*f)();
+{
+	register char *p, c;
+
+	while (p = *t++)
+		while (c = *p)
+			*p++ = (*f)(c);
+} */
+
+static
 tglob(c)
-register char c;
-#endif
+	register char c;
 {
 
 	if (any(c, globchars))
 		gflag |= c == '{' ? 2 : 1;
 	return (c);
 }
-
-int
-#ifdef __STDC__
-letter(register char c)
-#else
-letter(c)
-register char c;
-#endif
+/*
+static
+trim(c)
+	char c;
 {
 
-	return ((c >= 'a') && (c <= 'z') || (c >= 'A') && (c <= 'Z')
-		|| (c == '_'));
+	return (c & TRIM);
+} */
+
+
+letter(c)
+	register char c;
+{
+
+	return (c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' || c == '_');
 }
 
-int
-#ifdef __STDC__
-digit(register char c)
-#else
 digit(c)
-register char c;
-#endif
+	register char c;
 {
 
 	return (c >= '0' && c <= '9');
 }
 
-int
-#ifdef __STDC__
-any(register int c, register char *s)
-#else
-any(c,s)
-register int c;
-register char *s;
-#endif
+any(c, s)
+	register int c;
+	register char *s;
 {
 
 	while (*s)
@@ -688,14 +611,8 @@ register char *s;
 			return(1);
 	return(0);
 }
-
-int
-#ifdef __STDC__
-blklen(register char **av)
-#else
 blklen(av)
-register char **av;
-#endif
+	register char **av;
 {
 	register int i = 0;
 
@@ -705,12 +622,9 @@ register char **av;
 }
 
 char **
-#ifdef __STDC__
-blkcpy(char **oav, register char **bv)
-#else
 blkcpy(oav, bv)
-char **oav, **bv;
-#endif
+	char **oav;
+	register char **bv;
 {
 	register char **av = oav;
 
@@ -719,13 +633,8 @@ char **oav, **bv;
 	return (oav);
 }
 
-void
-#ifdef __STDC__
-blkfree(char **av0)
-#else
 blkfree(av0)
-char **av0;
-#endif
+	char **av0;
 {
 	register char **av = av0;
 
@@ -734,15 +643,10 @@ char **av0;
 }
 
 char *
-#ifdef __STDC__
-strspl(register char *cp, register char *dp)
-#else
 strspl(cp, dp)
-register char *cp, *dp;
-#endif
+	register char *cp, *dp;
 {
-	register char *ep = 
-	  (char *)malloc((unsigned)(strlen(cp) + strlen(dp) + 1));
+	register char *ep = malloc((unsigned)(strlen(cp) + strlen(dp) + 1));
 
 	if (ep == (char *)0)
 		fatal("Out of memory");
@@ -752,12 +656,8 @@ register char *cp, *dp;
 }
 
 char **
-#ifdef __STDC__
-copyblk(register char **v)
-#else
 copyblk(v)
-register char **v;
-#endif
+	register char **v;
 {
 	register char **nv = (char **)malloc((unsigned)((blklen(v) + 1) *
 						sizeof(char **)));
@@ -769,12 +669,8 @@ register char **v;
 
 static
 char *
-#ifdef __STDC__
-strend(register char *cp)
-#else
 strend(cp)
-register char *cp;
-#endif
+	register char *cp;
 {
 
 	while (*cp)
@@ -787,20 +683,13 @@ register char *cp;
  * user whose home directory is sought is currently.
  * We write the home directory of the user back there.
  */
-int
-#ifdef __STDC__
-gethdir(char *home)
-#else
 gethdir(home)
-char *home;
-#endif
+	char *home;
 {
 	register struct passwd *pp = getpwnam(home);
-	register char *root = NULL;
+
 	if (!pp || home + strlen(pp->pw_dir) >= lastgpathp)
 		return (1);
-	root = strstr(pp->pw_dir, "/./");
-        (void) strcpy(home, root?(root+2):pp->pw_dir);
-
+	(void) strcpy(home, pp->pw_dir);
 	return (0);
 }
